@@ -27,6 +27,9 @@
 --
 --
 -- $Log: combine_backup_files.c,v $
+-- Revision 1.8  2009/06/30 13:41:54  jsedwards
+-- Added include of new header.h file.
+--
 -- Revision 1.7  2009/03/08 00:43:49  jsedwards
 -- Changed include objectify_private.h to disk_io.h.
 --
@@ -57,6 +60,7 @@
 #include <string.h>
 
 #include "disk_io.h"
+#include "header.h"
 
 
 #define MAX_FILES 100
