@@ -1,6 +1,6 @@
 /*
 --             This file is part of the New World OS project
---                 Copyright (C) 2007-2008  QRW Software
+--                 Copyright (C) 2007-2009  QRW Software
 --           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --                      http://www.qrwsoftware.com
 --                      http://nwos.sourceforge.com
@@ -27,6 +27,9 @@
 --
 --
 -- $Log: device.c,v $
+-- Revision 1.8  2009/03/08 00:13:54  jsedwards
+-- Changed include objectify_private.h to objectify.h.
+--
 -- Revision 1.7  2008/09/01 00:10:50  jsedwards
 -- Fix copyright year.  NO code changes.
 --
@@ -60,7 +63,7 @@
 #include <string.h>   /* define memset */
 
 #include "crc32.h"
-#include "objectify_private.h"
+#include "objectify.h"
 
 #define MAX_NAME_SIZE 128
 #define MAX_NUMBER_SIZE 18
