@@ -27,12 +27,52 @@
 --
 --
 -- $Log: diff_compressed.c,v $
+-- Revision 1.25  2009/07/18 15:40:43  jsedwards
+-- Added code to set the upper 4 bytes of the reference in blocks from a 0029
+-- or older file to 0xfffffffe and back to zero if the block has to go around
+-- again.
+--
+-- Revision 1.24  2009/07/17 14:01:51  jsedwards
+-- Fixed to deal with having 0xfffffffe in the upper word of the reference in
+-- each block if the version > 29.
+--
+-- Revision 1.23  2009/07/17 13:23:19  jsedwards
+-- Fixed added blocks error upper bytes print statement to print to stderr too.
+--
+-- Revision 1.22  2009/07/17 13:10:01  jsedwards
+-- Fixed to report error and exit if upper four bytes of reference are not zero.
+--
+-- Revision 1.21  2009/07/11 13:30:30  jsedwards
+-- Changed to pass TYPE_CODE_COMPRESSED in the calls to check headers.
+--
+-- Revision 1.20  2009/07/10 15:05:59  jsedwards
+-- Changed to write "diff" into the headers of the output file if they are
+-- version 0030 or above.
+--
+-- Revision 1.19  2009/07/02 13:47:01  jsedwards
+-- Changed to call the new nwos_check_disk_header function instead of testing
+-- the header inline.
+--
+-- Revision 1.18  2009/04/06 14:21:14  jsedwards
+-- Merged in changes from CVS branch_0030_new_chunk_info branch, see revisions
+-- 1.15.2.1, 1.15.2.2, and 1.15.2.3 below.
+--
 -- Revision 1.17  2009/03/08 00:06:18  jsedwards
 -- Changed include objectify_private.h to disk_io.h.
 --
 -- Revision 1.16  2008/09/01 00:10:50  jsedwards
 -- Fix copyright year.  NO code changes.
 --
+-- Revision 1.15.2.3  2008/08/16 02:26:14  jsedwards
+-- Changed to work with new Disk_Header with total_chunks instead of
+-- total_blocks and still allow diff'ing with 0022 to 0029 headers.
+--
+-- Revision 1.15.2.2  2008/08/15 13:48:54  jsedwards
+-- Changed for new Disk_Header with total_chunks instead of total_blocks.
+--
+-- Revision 1.15.2.1  2008/08/15 13:39:16  jsedwards
+-- Removed old code to upgrade from version 0021 to 0022.
+--
 -- Revision 1.15  2008/01/04 04:25:08  jsedwards
 -- Added call to close the output file (fp3).
 --
@@ -93,7 +133,7 @@
 #include <string.h>
 #include <unistd.h>    /* define sleep() */
 
-#include "disk_io.h"
+#include "header.h"
 
 
 uint32 convert_4_uint8_to_uint32(uint8 byte[4])
@@ -138,11 +178,15 @@ int main(int argc, char* argv[])
     size_t write3;
     Disk_Header header1;
     Disk_Header header2;
-    Disk_Header_0021 old_header;
+    Disk_Header_0022_to_0029 old_header1;
+    Disk_Header_0022_to_0029 old_header2;
     bool wrapped_file = false;     /* true if older file that wasn't sequencial */
+    bool bad_upper_ref;
     int header1_version;
     int header2_version;
-
+    uint32 header1_chunks;
+    uint32 header2_chunks;
+    const char* error_msg;
 
     if (argc != 3 && argc != 4)
     {
@@ -178,9 +222,11 @@ int main(int argc, char* argv[])
 
     memcpy(&header1, buf1a, sizeof(header1));
 
-    if (memcmp(header1.magic_number, "NWOS", 4) != 0)
+    error_msg = nwos_check_disk_header(&header1, true, TYPE_CODE_COMPRESSED);  /* allow older versions */
+
+    if (error_msg != NULL)
     {
-	fprintf(stderr, "Not an Objectify file: %s\n", argv[1]);
+	fprintf(stderr, "%s: %s\n", error_msg, argv[1]);
 	fclose(fp1);
 	exit(1);
     }
@@ -214,11 +260,12 @@ int main(int argc, char* argv[])
 
     memcpy(&header2, buf2a, sizeof(header2));
 
-    if (memcmp(header2.magic_number, "NWOS", 4) != 0)
+    error_msg = nwos_check_disk_header(&header2, true, TYPE_CODE_COMPRESSED);  /* allow older versions */
+
+    if (error_msg != NULL)
     {
-	fprintf(stderr, "Not an Objectify file: %s\n", argv[2]);
+	fprintf(stderr, "%s: %s\n", error_msg, argv[1]);
 	fclose(fp1);
-	fclose(fp2);
 	exit(1);
     }
 
@@ -237,33 +284,6 @@ int main(int argc, char* argv[])
 	sleep(5);
     }
 
-    /* special code to handle upgrade from 0021 to 0022 */
-    if (memcmp(header1.version_string, "0021", 4) == 0 &&
-	memcmp(header2.version_string, "0022", 4) == 0)
-    {
-	memcpy(&old_header, buf1a, sizeof(old_header));
-
-	if (memcmp(old_header.total_private_blocks, header2.total_blocks, 4) != 0)
-	{
-	    fprintf(stderr, "Disk private blocks size different - %s: %u  %s: %u\n",
-		    argv[1], 
-		    convert_4_uint8_to_uint32(old_header.total_private_blocks),
-		    argv[2], 
-		    convert_4_uint8_to_uint32(header2.total_blocks));
-	}
-    }
-    else
-      {
-    if (memcmp(header1.total_blocks, header2.total_blocks, 4) != 0)
-    {
-	fprintf(stderr, "Disk private blocks size different - %s: %u  %s: %u\n",
-		argv[1], 
-		convert_4_uint8_to_uint32(header1.total_blocks),
-		argv[2], 
-		convert_4_uint8_to_uint32(header2.total_blocks));
-    }
-      }
-
     header1_version = version_string_to_int(header1.version_string);
 
     if (header1_version < 1)
@@ -280,18 +300,47 @@ int main(int argc, char* argv[])
 	exit(1);
     }
 
+    if (header1_version > header2_version)
+    {
+	fprintf(stderr, "ERROR: '%s' is a newer version than '%s', cannot diff!\n", argv[1], argv[2]);
+	exit(1);
+    }
+
     if (header1_version < 23 && header2_version < 23)
     {
 	wrapped_file = true;   /* it's an old file */
     }
-    else if (header1_version < 23 || header2_version < 23)
+    else if (header1_version < 23)
     {
-	fprintf(stderr, "ERROR: '%s' is an old wrapped file, but file '%s' is not, cannot diff!\n",
-		header1_version < 23 ? argv[1] : argv[2],
-		header1_version < 23 ? argv[2] : argv[1]);
+	fprintf(stderr, "ERROR: '%s' is an old wrapped file, but file '%s' is not, cannot diff!\n", argv[1], argv[2]);
 	exit(1);
     }
 
+    if (header1_version < 30)
+    {
+	memcpy(&old_header1, buf1a, sizeof(old_header1));
+	header1_chunks = convert_4_uint8_to_uint32(old_header1.total_blocks) / 65536; /* old chunks were 65536 blocks including bit map */
+    }
+    else
+    {
+	header1_chunks = convert_4_uint8_to_uint32(header1.total_chunks);
+    }
+
+    if (header2_version < 30)
+    {
+	memcpy(&old_header2, buf2a, sizeof(old_header2));
+	header2_chunks = convert_4_uint8_to_uint32(old_header2.total_blocks) / 65536; /* old chunks were 65536 blocks including bit map */
+    }
+    else
+    {
+	header2_chunks = convert_4_uint8_to_uint32(header2.total_chunks);
+    }
+
+    if (header1_chunks != header2_chunks)
+    {
+	fprintf(stderr, "Warning: disk total chunks size different - %s: %u  %s: %u\n", argv[1], header1_chunks, argv[2], header2_chunks);
+    }
+
     if (argc == 4)
     {
 	fp3 = fopen(argv[3], "w");
@@ -307,6 +356,11 @@ int main(int argc, char* argv[])
 
     if (fp3 != NULL)   /* write both headers into the patch file */
     {
+	if (header1_version >= 30)
+	{
+	    memcpy(&(((Disk_Header*)buf1a)->type_code), TYPE_CODE_DIFF, sizeof((((Disk_Header*)buf1a)->type_code)));
+	}
+	    
 	write3 = fwrite(buf1a, 1, sizeof(buf1a), fp3);
 
 	if (write3 != FILE_BLOCK_SIZE)
@@ -318,6 +372,11 @@ int main(int argc, char* argv[])
 	    exit(1);
 	}
 
+	if (header2_version >= 30)
+	{
+	    memcpy(&(((Disk_Header*)buf2a)->type_code), TYPE_CODE_DIFF, sizeof((((Disk_Header*)buf2a)->type_code)));
+	}
+	    
 	write3 = fwrite(buf2a, 1, sizeof(buf2a), fp3);
 
 	if (write3 != FILE_BLOCK_SIZE)
@@ -338,18 +397,61 @@ int main(int argc, char* argv[])
 
     while (!feof(fp1) && !feof(fp2) && read1 == FILE_BLOCK_SIZE && read2 == FILE_BLOCK_SIZE)
     {
-	if (buf1a[0] != 0 || buf1a[1] != 0 || buf1a[2] != 0 || buf1a[3] != 0)
+	bad_upper_ref = false;
+
+	if (header1_version < 30)
 	{
-	    printf("\n%s block %d - first four bytes not zero: %02x%02x%02x%02x\n",
-		   argv[1], block1, buf1a[0], buf1a[1], buf1a[2], buf1a[3]);
-	    break;
+	    if (buf1a[0] != 0 || buf1a[1] != 0 || buf1a[2] != 0 || buf1a[3] != 0)
+	    {
+		fprintf(stderr, "\n%s block %d - first four bytes not zero: %02x%02x%02x%02x\n",
+			argv[1], block1, buf1a[0], buf1a[1], buf1a[2], buf1a[3]);
+		bad_upper_ref = true;
+	    }
+
+	    if (header2_version >= 30)
+	    {
+		buf1a[0] = 0xff;
+		buf1a[1] = 0xff;
+		buf1a[2] = 0xff;
+		buf1a[3] = 0xfe;
+	    }
+	}
+	else 
+	{
+	    if (buf1a[0] != 0xff || buf1a[1] != 0xff || buf1a[2] != 0xff || buf1a[3] != 0xfe)
+	    {
+		fprintf(stderr, "\n%s block %d - first four bytes not fffffffe: %02x%02x%02x%02x\n",
+			argv[1], block1, buf1a[0], buf1a[1], buf1a[2], buf1a[3]);
+		bad_upper_ref = true;
+	    }
 	}
 
-	if (buf2a[0] != 0 || buf2a[1] != 0 || buf2a[2] != 0 || buf2a[3] != 0)
+	if (header2_version < 30)
 	{
-	    printf("\n%s block %d - first four bytes not zero: %02x%02x%02x%02x\n",
-		   argv[2], block2, buf2a[0], buf2a[1], buf2a[2], buf2a[3]);
-	    break;
+	    if (buf2a[0] != 0 || buf2a[1] != 0 || buf2a[2] != 0 || buf2a[3] != 0)
+	    {
+		fprintf(stderr, "\n%s block %d - first four bytes not zero: %02x%02x%02x%02x\n",
+			argv[2], block2, buf2a[0], buf2a[1], buf2a[2], buf2a[3]);
+		bad_upper_ref = true;
+	    }
+	}
+	else
+	{
+	    if (buf2a[0] != 0xff || buf2a[1] != 0xff || buf2a[2] != 0xff || buf2a[3] != 0xfe)
+	    {
+		fprintf(stderr, "\n%s block %d - first four bytes not fffffffe: %02x%02x%02x%02x\n",
+			argv[2], block2, buf2a[0], buf2a[1], buf2a[2], buf2a[3]);
+		bad_upper_ref = true;
+	    }
+	}
+
+	if (bad_upper_ref)
+	{
+	    fclose(fp1);
+	    fclose(fp2);
+	    if (fp3 != NULL) fclose(fp3);
+
+	    exit(1);
 	}
 
 	if (buf1a[4] == buf2a[4] && buf1a[5] == buf2a[5] && buf1a[6] == buf2a[6] && buf1a[7] == buf2a[7])
@@ -409,6 +511,15 @@ int main(int argc, char* argv[])
 	    }
 
 	    added++;
+
+	    /* Change the upper 4 bytes back because we are going through the loop again */
+	    if (header1_version < 30 && header2_version >= 30)
+	    {
+		buf1a[0] = 0;
+		buf1a[1] = 0;
+		buf1a[2] = 0;
+		buf1a[3] = 0;
+	    }
 	}
 
 	read2 = fread(buf2a, 1, sizeof(buf2a), fp2);
@@ -423,11 +534,34 @@ int main(int argc, char* argv[])
 
     while (!feof(fp2) && read2 == FILE_BLOCK_SIZE)
     {
-	if (buf2a[0] != 0 || buf2a[1] != 0 || buf2a[2] != 0 || buf2a[3] != 0)
+	bad_upper_ref = false;
+
+	if (header2_version < 30)
 	{
-	    printf("\n%s block %d - first four bytes not zero: %02x%02x%02x%02x\n",
-		   argv[2], block2, buf2a[0], buf2a[1], buf2a[2], buf2a[3]);
-	    break;
+	    if (buf2a[0] != 0 || buf2a[1] != 0 || buf2a[2] != 0 || buf2a[3] != 0)
+	    {
+		fprintf(stderr, "\n%s block %d - first four bytes not zero: %02x%02x%02x%02x\n",
+			argv[2], block2, buf2a[0], buf2a[1], buf2a[2], buf2a[3]);
+		bad_upper_ref = true;
+	    }
+	}
+	else
+	{
+	    if (buf2a[0] != 0xff || buf2a[1] != 0xff || buf2a[2] != 0xff || buf2a[3] != 0xfe)
+	    {
+		fprintf(stderr, "\n%s block %d - first four bytes not fffffffe: %02x%02x%02x%02x\n",
+			argv[2], block2, buf2a[0], buf2a[1], buf2a[2], buf2a[3]);
+		bad_upper_ref = true;
+	    }
+	}
+
+	if (bad_upper_ref)
+	{
+	    fclose(fp1);
+	    fclose(fp2);
+	    if (fp3 != NULL) fclose(fp3);
+
+	    exit(1);
 	}
 
 	printf("%02x%02x%02x%02x: added\n",
