@@ -27,6 +27,9 @@
 --
 --
 -- $Log: disk_usage.c,v $
+-- Revision 1.43  2009/07/21 12:12:50  jsedwards
+-- Fix Bug #2824303 - subtracted the 32-bit base from the chunk_info reference.
+--
 -- Revision 1.42  2009/07/12 18:17:10  jsedwards
 -- Changed variable that contain a number of blocks value to 64 bits and fixed
 -- the print statements for those variables to include 'll'.
@@ -378,7 +381,7 @@ void display_graph(Chunk_Info chunk_info[], uint32 num_chunks)
 
     for (i = 0; i < num_chunks; i++)
     {
-	segment = chunk_info[i].ref >> (32 - graph_bits);
+	segment = (chunk_info[i].ref - BASE_32_BIT_PRIVATE) >> (32 - graph_bits);
 
 	assert((RESERVED_PUBLIC_BLOCKS >> (32 - graph_bits)) <= segment && segment < (1 << graph_bits));
 
