@@ -1,9 +1,10 @@
 /*
---             This file is part of the New World OS project
---                 Copyright (C) 2006-2009  QRW Software
---           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
---                      http://www.qrwsoftware.com
---                      http://nwos.sourceforge.com
+--         This file is part of the New World OS and Objectify projects
+--                    Copyright (C) 2006-2009  QRW Software
+--              J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
+--                         http://www.qrwsoftware.com
+--                         http://nwos.sourceforge.com
+--                      http://objectify.sourceforge.com
 --
 --   This program is free software: you can redistribute it and/or modify
 --   it under the terms of the GNU General Public License as published by
@@ -27,9 +28,21 @@
 --
 --
 -- $Log: import_file.c,v $
+-- Revision 1.47  2009/05/09 10:48:44  jsedwards
+-- Merged in changes from alpha_29_10_branch (note: the same fix was already
+-- made in the main trunk in revision 1.45, so the merge only added the log
+-- message from the alpha_29_10_branch).
+--
 -- Revision 1.44.2.1  2009/05/05 13:13:16  jsedwards
 -- Added 'z' to a couple of print statements to eliminate compiler warning.
 --
+-- Revision 1.46  2009/04/10 14:25:14  jsedwards
+-- Changed to call new flush_to_disk function instead of the old flush_bit_maps
+-- function because the reference lists need to be flushed as well.
+--
+-- Revision 1.45  2009/04/05 12:42:35  jsedwards
+-- Fixed two compiler warnings in printf statements on amd64 machines.
+--
 -- Revision 1.44  2009/04/04 15:41:51  jsedwards
 -- Fix copyright years.  NO code changes.
 --
@@ -441,7 +454,7 @@ void process_file(const char* path, Add_Revision_Type add_revision, bool do_recu
 		break;
 	    }
 
-	    nwos_flush_bit_maps();
+	    nwos_flush_to_disk();
 	}
 	else  /* not enough space for this file */
 	{
