@@ -27,10 +27,33 @@
 --
 --
 -- $Log: prep_disk.c,v $
+-- Revision 1.75  2009/07/07 03:10:26  jsedwards
+-- Added code to write TYPE_CODE_FILE or TYPE_CODE_DISK to header.
+--
+-- Revision 1.74  2009/07/05 00:49:20  jsedwards
+-- Changed to include the new header.h file instead of the disk_io.h file.
+--
+-- Revision 1.73  2009/06/13 12:53:14  jsedwards
+-- Merged changes from Alpha_29.10 branch (version Alpha_29.11).
+--
 -- Revision 1.68.2.1  2009/06/06 15:14:52  jsedwards
 -- Fix Bug #2740516 - Added new get_response function that calls fgets and
 -- checks the return value.  Changed to call it instead of fgets.
 --
+-- Revision 1.72  2009/04/11 21:23:16  jsedwards
+-- Added include for the chunk_info.h file.
+--
+-- Revision 1.71  2009/04/11 13:47:59  jsedwards
+-- Fix so that total_chunks does not include the first chunk that stores the
+-- header and the chunk_info table.
+--
+-- Revision 1.70  2009/04/09 10:53:09  jsedwards
+-- Changed to calculate total blocks from total_chunks.
+--
+-- Revision 1.69  2009/04/07 12:00:01  jsedwards
+-- Merged changes from CVS branch_0030_new_chunk_info branch, see revisions
+-- 1.57.2.1 and 1.57.2.2 below.
+--
 -- Revision 1.68  2009/03/14 23:01:54  jsedwards
 -- Added include of the objectify.h file to define nwos_create_root.
 --
@@ -68,6 +91,13 @@
 -- Removed code and variables to read pass phrase and pass it to initialize,
 -- and change parameters passed to initialize.
 --
+-- Revision 1.57.2.2  2008/08/16 13:16:57  jsedwards
+-- Added logging the number of total and private chunks.
+--
+-- Revision 1.57.2.1  2008/08/13 03:51:23  jsedwards
+-- Changed for new chunks that store a full 16 megabytes and the new disk
+-- header with total_chunks instead of total_blocks.
+--
 -- Revision 1.57  2008/07/19 15:40:20  jsedwards
 -- Removed the #ifndef TEST_PASS_PHRASE around the verifying it that it is
 -- okay to write over the storage.  Too dangerous!!
@@ -306,8 +336,9 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
+#include "chunk_info.h"
 #include "config.h"
-#include "disk_io.h"
+#include "header.h"
 #include "log.h"
 #include "objectify.h"     /* define nwos_create_root */
 #include "user_config.h"
@@ -513,9 +544,8 @@ int main(int argc, char* argv[])
     bool do_private_blocks = true;
 
     uint32 total_blocks;
-    uint32 reserved_private_blocks;
+    uint32 total_chunks;
     uint32 blocks = 0;
-    uint32 num_chunks;
     Disk_Header disk_header;
     char msg[128];
     char* p;
@@ -595,14 +625,11 @@ int main(int argc, char* argv[])
     {
 	total_blocks = max_blocks_on_device(private_path);   /* zero means unlimited */
 
-	num_chunks = total_blocks / BLOCKS_IN_CHUNK;
-
-	/* round it to an even number of chunks */
-	total_blocks = num_chunks * BLOCKS_IN_CHUNK;
+	total_chunks = total_blocks / BLOCKS_IN_CHUNK - 1;   /* one chunk is for the system (header and chunk info table) */
     }
     else
     {
-	total_blocks = 2 * BLOCKS_IN_CHUNK;	     /* start with enough space for the header and one private chunk */
+	total_chunks = 1;	     /* start with enough space for one private chunk */
     }
 
     if (getenv(TEST_ENVIRONMENT_VARIABLE) == NULL)
@@ -610,10 +637,8 @@ int main(int argc, char* argv[])
 	agree_no_warranty();   /* make sure he or she agrees with the licensing */
     }
 
-    reserved_private_blocks = total_blocks - BLOCKS_IN_CHUNK;  /* first chunk is for header */
-
-    printf("\ntotal chunks: %u  private chunks: %u\n",
-	   total_blocks / BLOCKS_IN_CHUNK, reserved_private_blocks / BLOCKS_IN_CHUNK);
+    snprintf(msg, sizeof(msg), "total chunks: %u  private chunks: %u", total_chunks + 1, total_chunks);
+    printf("\n%s\n", msg);
 
 
     chunk = malloc(CHUNK_SIZE);
@@ -640,6 +665,7 @@ int main(int argc, char* argv[])
 	exit(1);
     }
 
+
     /***************************/
     /* Open the storage device */
     /***************************/
@@ -682,19 +708,22 @@ int main(int argc, char* argv[])
     memcpy(disk_header.magic_number, MAGIC_NUMBER, 4);
     memcpy(disk_header.version_string, VERSION_STRING, 4);
 
+    if (is_file)
+    {
+	memcpy(disk_header.type_code, TYPE_CODE_FILE, 4);
+    }
+    else
+    {
+	memcpy(disk_header.type_code, TYPE_CODE_DISK, 4);
+    }
+
     nwos_get_time_stamp(disk_header.last_prep_disk);
 
     memcpy(disk_header.last_change, disk_header.last_prep_disk, sizeof(disk_header.last_change));
 
-    nwos_uint32_to_4_uint8(&reserved_private_blocks, disk_header.total_blocks);
+    nwos_uint32_to_4_uint8(&total_chunks, disk_header.total_chunks);
 
 
-    /* compute the number of private blocks used */
-
-    assert(reserved_private_blocks % BLOCKS_IN_CHUNK == 0);
-
-    num_chunks = reserved_private_blocks / BLOCKS_IN_CHUNK;  /* round up */
-
     /* set the number of used blocks to 0 */
     blocks = 0;
     nwos_uint32_to_4_uint8(&blocks, disk_header.used_blocks);
@@ -726,12 +755,6 @@ int main(int argc, char* argv[])
     {
 	memset(chunk, 0, CHUNK_SIZE);
 
-	/* private blocks have bit maps so mark off 32 bits for the bit map itself */
-	chunk[0] = 0xff;
-	chunk[1] = 0xff;
-	chunk[2] = 0xff;
-	chunk[3] = 0xff;
-
 	if (write(obj_fd, chunk, CHUNK_SIZE) != CHUNK_SIZE)
 	{
 	    perror(private_path);
@@ -747,13 +770,9 @@ int main(int argc, char* argv[])
 
     free(chunk);
 
-    printf("Total chunks: %u\n", num_chunks);
-
-
-    snprintf(msg, sizeof(msg), "   private blocks: %9u  used: %9u", reserved_private_blocks, blocks);
+    snprintf(msg, sizeof(msg), "   private blocks: %9u  used: %9u", total_chunks * USABLE_BLOCKS_PER_CHUNK, blocks);
     nwos_log(msg);
 
-
     if (do_private_blocks)
     {
 	if (getenv(TEST_ENVIRONMENT_VARIABLE) == NULL)
