@@ -29,6 +29,9 @@
 #  This script runs a quick test of file importing and exporting.
 # 
 #  $Log: test_diff_compressed.sh,v $
+#  Revision 1.3  2009/07/21 13:38:03  jsedwards
+#  Partial fix of Bug #2824308 - added test of sync_archive.
+#
 #  Revision 1.2  2009/07/18 04:14:03  jsedwards
 #  Fix wrong name in print (echo) statement.
 #
@@ -321,9 +324,70 @@ if ! $EXEC/verify_file --remove-no-shred *; then
    exit 1
 fi
 
+
+
+echo "---------------------------"
+echo " prep a second archive..."
+echo "---------------------------"
+
+export OBJECTIFY_PRIVATE_PATH=/tmp/objectify_test/private2.obj
+
+if ! $EXEC/prep_disk --no-private; then
+   echo ""
+   echo "prep_disk failed"
+   echo ""
+   exit 1
+fi
+
+sleep 2
+
+
+echo "---------------------"
+echo " sync archive..."
+echo "---------------------"
+if ! $EXEC/sync_archive /tmp/objectify_test/private.obj /tmp/objectify_test/private2.obj; then
+   echo ""
+   echo "sync_archive failed"
+   echo ""
+   exit 1
+fi
+
+sleep 2
+
+
+echo "------------------------------"
+echo " compress second archive..."
+echo "------------------------------"
+if ! $EXEC/compress_sparse /tmp/objectify_test/compressed-second; then
+   echo ""
+   echo "compress_sparse failed"
+   echo ""
+   exit 1
+fi
+
+
+sleep 2
+
+echo "-----------------------------------------------------"
+echo " verify first and second compressed are the same..."
+echo "-----------------------------------------------------"
+if ! $EXEC/diff_compressed /tmp/objectify_test/compressed-after /tmp/objectify_test/compressed-second; then
+   echo ""
+   echo "first and second compressed files differ"
+   echo ""
+   exit 1
+fi
+
+echo "PLEASE check above that there were no differences!"
+
+sleep 2
+
+
+
 echo ""
 echo "All tests passed!"
 echo ""
 
+
 exit 0
 
