@@ -27,6 +27,11 @@
 --
 --
 -- $Log: url.c,v $
+-- Revision 1.8  2009/07/19 13:21:44  jsedwards
+-- Added ifdef VERIFY_WRITE around code that reads back objects to verify they
+-- were written correctly, they don't work correctly when nwos_write_block
+-- modifies the upper 4 bytes of the reference in the block.
+--
 -- Revision 1.7  2009/03/10 11:53:43  jsedwards
 -- Moved MAX_URL_SIZE to the objectify.h file and renamed to MAX_URL_OBJ_SIZE.
 --
@@ -171,6 +176,7 @@ ObjCreateResult nwos_create_url(char* url, ObjRef* ref)
 
 	nwos_add_to_references(ref, &public_url_class_ref);
 
+#ifdef VERIFY_WRITE
 	ptr_url_obj = malloc(sizeof(C_struct_Uniform_Resource_Locator) + length);
 	assert(nwos_read_object_from_disk(ref, ptr_url_obj, sizeof(C_struct_Uniform_Resource_Locator) + length));
 	assert(memcmp(kludge, ptr_url_obj, sizeof(C_struct_Uniform_Resource_Locator) + length) == 0);
@@ -181,6 +187,7 @@ ObjCreateResult nwos_create_url(char* url, ObjRef* ref)
 
 	free(ptr_url_obj);
 	ptr_url_obj = NULL;
+#endif
 
 	result = CREATED_NEW;
     }
